# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys

from flask import Flask
from oslo_log import log

from almanach.api import auth_adapter
from almanach.api.v1 import routes
from almanach.core import controller
from almanach.core import opts
from almanach.storage import storage_driver

LOG = log.getLogger(__name__)

app = Flask('almanach')
app.register_blueprint(routes.api)


def main():
    try:
        opts.CONF(sys.argv[1:])
        config = opts.CONF

        database_driver = storage_driver.StorageDriver(config).get_database_driver()
        database_driver.connect()

        routes.controller = controller.Controller(config, database_driver)
        routes.auth_adapter = auth_adapter.AuthenticationAdapter(config).get_authentication_adapter()

        LOG.info('Listening on %s:%d', config.api.bind_ip, config.api.bind_port)
        app.run(host=config.api.bind_ip, port=config.api.bind_port)
    except Exception as e:
        LOG.exception(e)
        sys.exit(100)

if __name__ == '__main__':
    main()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
import pymongo

from almanach.core import exception
from almanach.core import model
from almanach.core.model import build_entity_from_dict
from almanach.storage.drivers import base_driver

LOG = log.getLogger(__name__)


class MongoDbDriver(base_driver.BaseDriver):
    def __init__(self, config, db=None):
        super(MongoDbDriver, self).__init__(config)
        self.db = db

    def connect(self):
        connection = pymongo.MongoClient(self.config.database.connection_url, tz_aware=True)
        connection_options = pymongo.uri_parser.parse_uri(self.config.database.connection_url)
        self.db = connection[connection_options['database']]

    def get_active_entity(self, entity_id):
        entity = self._get_one_entity_from_db({"entity_id": entity_id, "end": None})
        if not entity:
            raise KeyError("Unable to find entity id %s" % entity_id)
        return build_entity_from_dict(entity)

    def count_entities(self):
        return self.db.entity.count()

    def count_active_entities(self):
        return self.db.entity.find({"end": None}).count()

    def count_entity_entries(self, entity_id):
        return self.db.entity.find({"entity_id": entity_id}).count()

    def has_active_entity(self, entity_id):
        return self.db.entity.find({"entity_id": entity_id, "end": None}).count() == 1

    def list_entities(self, project_id, start, end, entity_type=None):
        args = {"project_id": project_id, "start": {"$lte": end}, "$or": [{"end": None}, {"end": {"$gte": start}}]}
        if entity_type:
            args["entity_type"] = entity_type
        entities = self._get_entities_from_db(args)
        return [build_entity_from_dict(entity) for entity in entities]

    def get_all_entities_by_id(self, entity_id):
        entities = self.db.entity.find({"entity_id": entity_id}, {"_id": 0})
        return [build_entity_from_dict(entity) for entity in entities]

    def list_entities_by_id(self, entity_id, start, end):
        entities = self.db.entity.find({"entity_id": entity_id,
                                        "start": {"$gte": start},
                                        "$and": [
                                            {"end": {"$ne": None}},
                                            {"end": {"$lte": end}}
                                        ]
                                        }, {"_id": 0})
        return [build_entity_from_dict(entity) for entity in entities]

    def update_closed_entity(self, entity, data):
        self.db.entity.update({"entity_id": entity.entity_id, "start": entity.start, "end": entity.end},
                              {"$set": data})

    def insert_entity(self, entity):
        self._insert_entity(entity.as_dict())

    def insert_volume_type(self, volume_type):
        self.db.volume_type.insert(volume_type.__dict__)

    def get_volume_type(self, volume_type_id):
        volume_type = self.db.volume_type.find_one({"volume_type_id": volume_type_id})
        if not volume_type:
            raise exception.VolumeTypeNotFoundException(volume_type_id=volume_type_id)

        return model.VolumeType(volume_type_id=volume_type["volume_type_id"],
                                volume_type_name=volume_type["volume_type_name"])

    def delete_volume_type(self, volume_type_id):
        if volume_type_id is None:
            raise exception.AlmanachException("Trying to delete all volume types which is not permitted.")
        returned_value = self.db.volume_type.remove({"volume_type_id": volume_type_id})
        if returned_value['n'] == 1:
            LOG.info("Deleted volume type with id '%s' successfully.", volume_type_id)
        else:
            raise exception.AlmanachException(
                    "Volume type with id {} doesn't exist in the database.".format(volume_type_id))

    def list_volume_types(self):
        volume_types = self.db.volume_type.find()
        return [model.VolumeType(volume_type_id=volume_type["volume_type_id"],
                                 volume_type_name=volume_type["volume_type_name"]) for volume_type in volume_types]

    def close_active_entity(self, entity_id, end):
        self.db.entity.update({"entity_id": entity_id, "end": None}, {"$set": {"end": end, "last_event": end}})

    def update_active_entity(self, entity):
        self.db.entity.update({"entity_id": entity.entity_id, "end": None}, {"$set": entity.as_dict()})

    def delete_active_entity(self, entity_id):
        self.db.entity.remove({"entity_id": entity_id, "end": None})

    def _insert_entity(self, entity):
        self.db.entity.insert(entity)

    def _get_entities_from_db(self, args):
        return list(self.db.entity.find(args, {"_id": 0}))

    def _get_one_entity_from_db(self, args):
        return self.db.entity.find_one(args, {"_id": 0})



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that
from hamcrest import calling
from hamcrest import equal_to
from hamcrest import raises

from almanach.api.auth import private_key_auth
from almanach.core import exception

from tests import base


class PrivateKeyAuthenticationTest(base.BaseTestCase):

    def setUp(self):
        super(PrivateKeyAuthenticationTest, self).setUp()
        self.auth_backend = private_key_auth.PrivateKeyAuthentication("my token")

    def test_with_correct_token(self):
        assert_that(self.auth_backend.validate("my token"), equal_to(True))

    def test_with_invalid_token(self):
        assert_that(calling(self.auth_backend.validate)
                    .with_args("bad token"), raises(exception.AuthenticationFailureException))

    def test_with_empty_token(self):
        assert_that(calling(self.auth_backend.validate)
                    .with_args(None), raises(exception.AuthenticationFailureException))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from hamcrest import assert_that
from hamcrest import instance_of

from almanach.api.auth import keystone_auth
from almanach.api.auth import mixed_auth
from almanach.api.auth import private_key_auth
from almanach.api import auth_adapter

from tests import base


class AuthenticationAdapterTest(base.BaseTestCase):
    def setUp(self):
        super(AuthenticationAdapterTest, self).setUp()
        self.auth_adapter = auth_adapter.AuthenticationAdapter(self.config)

    def test_assert_that_the_default_backend_is_private_key(self):
        adapter = self.auth_adapter.get_authentication_adapter()
        assert_that(adapter, instance_of(private_key_auth.PrivateKeyAuthentication))

    def test_get_keystone_auth_backend(self):
        self.config_fixture.config(strategy='keystone', group='auth')
        adapter = self.auth_adapter.get_authentication_adapter()
        assert_that(adapter, instance_of(keystone_auth.KeystoneAuthentication))

    def test_get_mixed_auth_backend(self):
        self.config_fixture.config(strategy='token,keystone', group='auth')
        adapter = self.auth_adapter.get_authentication_adapter()
        assert_that(adapter, instance_of(mixed_auth.MixedAuthentication))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import kombu
from oslo_log import log
import sys

from almanach.collector import bus_adapter
from almanach.collector import retry_adapter
from almanach.core import controller
from almanach.core import opts
from almanach.storage import storage_driver

LOG = log.getLogger(__name__)


def main():
    try:
        opts.CONF(sys.argv[1:])
        config = opts.CONF

        database_driver = storage_driver.StorageDriver(config).get_database_driver()
        database_driver.connect()

        application_controller = controller.Controller(config, database_driver)
        connection = kombu.Connection(hostname=config.collector.rabbit_host,
                                      port=config.collector.rabbit_port,
                                      userid=config.collector.rabbit_username,
                                      password=config.collector.rabbit_password,
                                      heartbeat=540)
        retry_listener = retry_adapter.RetryAdapter(config, connection)
        bus_listener = bus_adapter.BusAdapter(config, application_controller,
                                              connection, retry_listener)

        LOG.info('Listening for incoming events')
        bus_listener.run()
    except Exception as e:
        LOG.exception(e)
        sys.exit(100)

if __name__ == '__main__':
    main()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging

LOG = logging.getLogger(__name__)
CONF = cfg.CONF
DOMAIN = 'almanach'

database_opts = [
    cfg.StrOpt('driver',
               default='mongodb',
               help='Database driver'),
    cfg.StrOpt('connection_url',
               default='mongodb://almanach:almanach@localhost:27017/almanach',
               help='Database connection URL'),
]

api_opts = [
    cfg.IPOpt('bind_ip',
              default='127.0.0.1',
              help='IP address to listen on'),
    cfg.PortOpt('bind_port',
                default=8000,
                help='TCP port number to listen on'),
]

collector_opts = [
    cfg.HostnameOpt('rabbit_host',
                    default='localhost',
                    help='RabbitMQ Hostname'),
    cfg.PortOpt('rabbit_port',
                default=5672,
                help='RabbitMQ TCP port'),
    cfg.StrOpt('rabbit_username',
               help='RabbitMQ Username'),
    cfg.StrOpt('rabbit_password',
               help='RabbitMQ Password'),
    cfg.StrOpt('queue',
               default='almanach.info',
               help='Default queue name'),
    cfg.StrOpt('exchange',
               default='almanach.info',
               help='Default exchange name'),
    cfg.StrOpt('routing_key',
               default='almanach.info',
               help='Default queue routing key'),
    cfg.StrOpt('retry_queue',
               default='almanach.retry',
               help='Retry queue name'),
    cfg.StrOpt('retry_exchange',
               default='almanach.retry',
               help='Retry exchange name'),
    cfg.StrOpt('retry_return_exchange',
               default='almanach',
               help='Retry return exchange name'),
    cfg.IntOpt('retry_ttl',
               default=10,
               help='Time to live value of messages sent on the retry queue'),
    cfg.IntOpt('max_retries',
               default=3,
               help='Maximal number of message retries'),
    cfg.StrOpt('dead_queue',
               default='almanach.dead',
               help='Dead queue name'),
    cfg.StrOpt('dead_exchange',
               default='almanach.dead',
               help='Dead exchange name'),
]

auth_opts = [
    cfg.StrOpt('strategy',
               default='private_key',
               help='Authentication driver for the API'),
    cfg.StrOpt('private_key',
               default='secret',
               help='Private key for private key authentication'),
    cfg.StrOpt('keystone_username',
               help='Keystone service username'),
    cfg.StrOpt('keystone_password',
               help='Keystone service password'),
    cfg.StrOpt('keystone_tenant',
               help='Keystone service tenant'),
    cfg.StrOpt('keystone_url',
               default='http://keystone_url:5000/v2.0',
               help='Keystone URL'),
]

resource_opts = [
    cfg.IntOpt('volume_existence_threshold',
               default=60,
               help='Volume existence threshold'),
    cfg.ListOpt('device_metadata_whitelist',
                default=[],
                deprecated_for_removal=True,
                help='Metadata to include in entity'),
]

CONF.register_opts(database_opts, group='database')
CONF.register_opts(api_opts, group='api')
CONF.register_opts(collector_opts, group='collector')
CONF.register_opts(auth_opts, group='auth')
CONF.register_opts(resource_opts, group='resources')

logging.register_options(CONF)
logging.setup(CONF, DOMAIN)


def list_opts():
    return [
        ('database', database_opts),
        ('api', api_opts),
        ('collector', collector_opts),
        ('api.auth', auth_opts),
        ('resources', resource_opts),
    ]



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class AlmanachException(Exception):
    def __init__(self, message=None):
        self.message = message


class AlmanachEntityNotFoundException(AlmanachException):
    pass


class AuthenticationFailureException(AlmanachException):
    pass


class DateFormatException(AlmanachException):
    def __init__(self, message=None):
        if not message:
            message = "The provided date has an invalid format. Format should be of yyyy-mm-ddThh:mm:ss.msZ, " \
                      "ex: 2015-01-31T18:24:34.1523Z"

        super(DateFormatException, self).__init__(message)


class MultipleEntitiesMatchingQueryException(AlmanachException):
    pass


class InvalidAttributeException(AlmanachException):
    def __init__(self, errors):
        self.errors = errors

    def get_error_message(self):
        messages = {}
        for error in self.errors:
            messages[error.path[0]] = error.msg

        return messages


class VolumeTypeNotFoundException(AlmanachException):
    def __init__(self, volume_type_id, message=None):
        if not message:
            message = "Unable to find volume_type id '{volume_type_id}'".format(volume_type_id=volume_type_id)

        super(VolumeTypeNotFoundException, self).__init__(message)


class DatabaseDriverNotSupportedException(AlmanachException):
    pass



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.core import exception
from almanach.storage.drivers import mongodb_driver

LOG = log.getLogger(__name__)


class StorageDriver(object):
    def __init__(self, config):
        self.config = config

    def get_database_driver(self):
        if self.config.database.driver == 'mongodb':
            LOG.info('Loading MongoDb storage driver')
            return mongodb_driver.MongoDbDriver(self.config)
        raise exception.DatabaseDriverNotSupportedException(
                'Unknown database driver {}'.format(self.config.database.driver))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.api.auth import base_auth
from almanach.core import exception

LOG = log.getLogger(__name__)


class MixedAuthentication(base_auth.BaseAuth):
    def __init__(self, authentication_adapters):
        self.authentication_adapters = authentication_adapters

    def validate(self, token):
        for adapter in self.authentication_adapters:
            try:
                valid = adapter.validate(token)
                if valid:
                    LOG.debug('Validated token with adapter: %s', adapter.__class__)
                    return True
            except exception.AuthenticationFailureException:
                LOG.debug('Failed to validate with adapter: %s', adapter.__class__)
        raise exception.AuthenticationFailureException('Unable to authenticate against any authentication adapters')



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import voluptuous

from almanach.core import exception


class InstanceValidator(object):
    def __init__(self):
        self.schema = voluptuous.Schema({
            'name': six.text_type,
            'flavor': six.text_type,
            'os': {
                voluptuous.Required('distro'): six.text_type,
                voluptuous.Required('version'): six.text_type,
                voluptuous.Required('os_type'): six.text_type,
            },
            'metadata': dict,
            'start_date': voluptuous.Datetime(),
            'end_date': voluptuous.Datetime(),
        })

    def validate_update(self, payload):
        try:
            return self.schema(payload)
        except voluptuous.MultipleInvalid as e:
            raise exception.InvalidAttributeException(e.errors)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from flexmock import flexmock_teardown
from oslo_config import fixture
import testtools

from almanach.core import opts


class BaseTestCase(testtools.TestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.config_fixture = self.useFixture(fixture.Config(conf=opts.CONF))
        self.config = self.config_fixture.conf

    def tearDown(self):
        super(BaseTestCase, self).tearDown()
        flexmock_teardown()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.api.auth import base_auth
from almanach.core import exception


class PrivateKeyAuthentication(base_auth.BaseAuth):
    def __init__(self, private_key):
        self.private_key = private_key

    def validate(self, token):
        if token is None or self.private_key != token:
            raise exception.AuthenticationFailureException("Invalid Token")
        return True



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import kombu

from oslo_log import log

LOG = log.getLogger(__name__)


class RetryAdapter(object):
    def __init__(self, config, connection):
        self.config = config
        self.connection = connection

        retry_exchange = self._configure_retry_exchanges(self.connection)
        dead_exchange = self._configure_dead_exchange(self.connection)

        self._retry_producer = kombu.Producer(self.connection, exchange=retry_exchange)
        self._dead_producer = kombu.Producer(self.connection, exchange=dead_exchange)

    def publish_to_dead_letter(self, message):
        death_count = self._rejected_count(message)
        LOG.info('Message die %d times', death_count)

        if death_count < self.config.collector.max_retries:
            LOG.info('Publishing message to retry queue')
            self._publish_message(self._retry_producer, message)
        else:
            LOG.info('Publishing message to dead letter queue')
            self._publish_message(self._dead_producer, message)

    def _configure_retry_exchanges(self, connection):
        def declare_queues():
            channel = connection.channel()

            retry_exchange = kombu.Exchange(
                name=self.config.collector.retry_exchange,
                type='direct',
                channel=channel)

            retry_queue = kombu.Queue(
                name=self.config.collector.retry_queue,
                exchange=retry_exchange,
                routing_key=self.config.collector.routing_key,
                queue_arguments=self._get_queue_arguments(),
                channel=channel)

            main_exchange = kombu.Exchange(
                    name=self.config.collector.retry_return_exchange,
                    type='direct',
                    channel=channel)

            main_queue = kombu.Queue(
                name=self.config.collector.queue,
                exchange=main_exchange,
                durable=False,
                routing_key=self.config.collector.routing_key,
                channel=channel)

            retry_queue.declare()
            main_queue.declare()

            return retry_exchange

        def error_callback(exception, interval):
            LOG.error('Failed to declare queues and exchanges, retrying in %d seconds. %r', interval, exception)

        declare_queues = connection.ensure(connection, declare_queues, errback=error_callback,
                                           interval_start=0, interval_step=5, interval_max=30)
        return declare_queues()

    def _configure_dead_exchange(self, connection):
        def declare_dead_queue():
            channel = connection.channel()
            dead_exchange = kombu.Exchange(
                name=self.config.collector.dead_exchange,
                type='direct',
                channel=channel)

            dead_queue = kombu.Queue(
                name=self.config.collector.dead_queue,
                routing_key=self.config.collector.routing_key,
                exchange=dead_exchange,
                channel=channel)

            dead_queue.declare()

            return dead_exchange

        def error_callback(exception, interval):
            LOG.error('Failed to declare dead queue and exchange, retrying in %d seconds. %r',
                      interval, exception)

        declare_dead_queue = connection.ensure(connection, declare_dead_queue, errback=error_callback,
                                               interval_start=0, interval_step=5, interval_max=30)
        return declare_dead_queue()

    def _get_queue_arguments(self):
        return {
            "x-message-ttl": self.config.collector.retry_ttl * 1000,
            "x-dead-letter-exchange": self.config.collector.retry_return_exchange,
            "x-dead-letter-routing-key": self.config.collector.routing_key,
        }

    def _rejected_count(self, message):
        if 'x-death' in message.headers:
            return len(message.headers['x-death'])
        return 0

    def _publish_message(self, producer, message):
        publish = self.connection.ensure(producer, producer.publish, errback=self._error_callback,
                                         interval_start=0, interval_step=5, interval_max=30)
        publish(message.body,
                routing_key=message.delivery_info['routing_key'],
                headers=message.headers,
                content_type=message.content_type,
                content_encoding=message.content_encoding)

    def _error_callback(self, exception, interval):
        LOG.error('Failed to publish message to dead letter queue, retrying in %d seconds. %r',
                  interval, exception)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import flask
from flexmock import flexmock
import json
import oslo_serialization

from almanach.api.v1 import routes
from almanach.core import exception
from tests import base


class BaseApi(base.BaseTestCase):

    def setUp(self):
        super(BaseApi, self).setUp()
        self.prepare()
        self.prepare_with_successful_authentication()

    def prepare(self):
        self.controller = flexmock()
        self.auth_adapter = flexmock()
        routes.controller = self.controller
        routes.auth_adapter = self.auth_adapter

        self.app = flask.Flask("almanach")
        self.app.register_blueprint(routes.api)

    def prepare_with_successful_authentication(self):
        self.auth_adapter.should_receive("validate").and_return(True)

    def prepare_with_failed_authentication(self):
        self.auth_adapter.should_receive("validate")\
            .and_raise(exception.AuthenticationFailureException("Wrong credentials"))

    def api_get(self, url, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "get", None, query_string, headers, accept)

    def api_head(self, url, headers):
        return self._api_call(url=url, method="head", headers=headers, accept='application/json')

    def api_post(self, url, data=None, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "post", data, query_string, headers, accept)

    def api_put(self, url, data=None, query_string=None, headers=None, accept='application/json'):
        return self._api_call(url, "put", data, query_string, headers, accept)

    def api_delete(self, url, query_string=None, data=None, headers=None, accept='application/json'):
        return self._api_call(url, "delete", data, query_string, headers, accept)

    def _api_call(self, url, method, data=None, query_string=None, headers=None, accept='application/json'):
        with self.app.test_client() as http_client:
            if not headers:
                headers = {}
        headers['Accept'] = accept
        result = getattr(http_client, method)(url, data=json.dumps(data), query_string=query_string, headers=headers)
        return_data = oslo_serialization.jsonutils.loads(result.data) \
            if result.headers.get('Content-Type') == 'application/json' \
            else result.data
        return result.status_code, return_data


class DateMatcher(object):
    def __init__(self, date):
        self.date = date

    def __eq__(self, other):
        return other == self.date

    def __ne__(self, other):
        return not self.__eq__(other)


def a_date_matching(date_string):
    return DateMatcher(datetime.strptime(date_string, "%Y-%m-%d %H:%M:%S.%f"))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.api.auth import keystone_auth
from almanach.api.auth import mixed_auth
from almanach.api.auth import private_key_auth

LOG = log.getLogger(__name__)


class AuthenticationAdapter(object):
    def __init__(self, config):
        self.config = config

    def get_authentication_adapter(self):
        if self.config.auth.strategy == 'keystone':
            return self._get_keystone_auth()
        elif all(auth_method in self.config.auth.strategy for auth_method in ['token', 'keystone']):
            adapters = [self._get_private_key_auth(), self._get_keystone_auth()]
            return mixed_auth.MixedAuthentication(adapters)
        return self._get_private_key_auth()

    def _get_private_key_auth(self):
        LOG.info('Loading PrivateKey authentication adapter')
        return private_key_auth.PrivateKeyAuthentication(self.config.auth.private_key)

    def _get_keystone_auth(self):
        LOG.info('Loading Keystone authentication backend')
        token_manager = keystone_auth.KeystoneTokenManagerFactory(self.config)
        return keystone_auth.KeystoneAuthentication(token_manager)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import kombu
import six

from kombu import mixins
from oslo_log import log
from oslo_serialization import jsonutils

from almanach.collector.handlers import instance_handler
from almanach.collector.handlers import volume_handler

LOG = log.getLogger(__name__)


class BusAdapter(mixins.ConsumerMixin):
    def __init__(self, config, controller, connection, retry_adapter):
        super(BusAdapter, self).__init__()
        self.config = config
        self.controller = controller
        self.connection = connection
        self.retry_adapter = retry_adapter

    def on_message(self, notification, message):
        try:
            self._process_notification(notification)
        except Exception as e:
            LOG.warning('Sending notification to retry letter exchange %s', jsonutils.dumps(notification))
            LOG.exception(e)
            self.retry_adapter.publish_to_dead_letter(message)
        message.ack()

    def _process_notification(self, notification):
        if isinstance(notification, six.string_types):
            notification = jsonutils.loads(notification)

        event_type = notification.get('event_type')
        LOG.info('Received event: %s', event_type)

        instance_handler.InstanceHandler(self.controller).handle_events(event_type, notification)
        volume_handler.VolumeHandler(self.controller).handle_events(event_type, notification)

    def get_consumers(self, consumer, channel):
        queue = kombu.Queue(self.config.collector.queue, routing_key=self.config.collector.routing_key)
        return [consumer(
            [queue],
            callbacks=[self.on_message],
            auto_declare=False)]

    def run(self, _tokens=1):
        try:
            super(BusAdapter, self).run(_tokens)
        except KeyboardInterrupt:
            pass



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.core import exception
from almanach.storage.drivers import mongodb_driver
from almanach.storage import storage_driver

from tests import base


class StorageDriverTest(base.BaseTestCase):
    def setUp(self):
        super(StorageDriverTest, self).setUp()
        self.storage_driver = storage_driver.StorageDriver(self.config)

    def test_get_default_database_adapter(self):
        self.assertIsInstance(self.storage_driver.get_database_driver(), mongodb_driver.MongoDbDriver)

    def test_get_unknown_database_adapter(self):
        self.config_fixture.config(driver='foobar', group='database')
        self.assertRaises(exception.DatabaseDriverNotSupportedException, self.storage_driver.get_database_driver)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class BaseDriver(object):

    def __init__(self, config):
        self.config = config

    @abc.abstractmethod
    def connect(self):
        pass

    @abc.abstractmethod
    def get_active_entity(self, entity_id):
        pass

    @abc.abstractmethod
    def count_entities(self):
        pass

    @abc.abstractmethod
    def count_active_entities(self):
        pass

    @abc.abstractmethod
    def count_entity_entries(self, entity_id):
        pass

    @abc.abstractmethod
    def has_active_entity(self, entity_id):
        pass

    @abc.abstractmethod
    def list_entities(self, project_id, start, end, entity_type=None):
        pass

    @abc.abstractmethod
    def get_all_entities_by_id(self, entity_id):
        pass

    @abc.abstractmethod
    def list_entities_by_id(self, entity_id, start, end):
        pass

    @abc.abstractmethod
    def update_closed_entity(self, entity, data):
        pass

    @abc.abstractmethod
    def insert_entity(self, entity):
        pass

    @abc.abstractmethod
    def close_active_entity(self, entity_id, end):
        pass

    @abc.abstractmethod
    def update_active_entity(self, entity):
        pass

    @abc.abstractmethod
    def delete_active_entity(self, entity_id):
        pass

    @abc.abstractmethod
    def insert_volume_type(self, volume_type):
        pass

    @abc.abstractmethod
    def get_volume_type(self, volume_type_id):
        pass

    @abc.abstractmethod
    def delete_volume_type(self, volume_type_id):
        pass

    @abc.abstractmethod
    def list_volume_types(self):
        pass
